﻿using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;

namespace Feel
{
	public class EnjoyGlassBody : MVRScript {

        private EnjoyGlassBodyUI _enjoyGlassBodyUI;
        private FeelTriggersControl _trigger_control;

        private Atom _girl_atom;

        private int _initLoops = 0;
        private int _maxInitLoops = 1000;

        public Transform ObjectRoot;

        private List<FeelTrigger> pose_triggers;
        private FeelTrigger _pov_trigger;
        private FeelTrigger _start_trigger;
        private FeelTrigger _end_trigger;
        private FeelTrigger _vr_on_trigger;
        private FeelTrigger _vr_off_trigger;

        public override void Init() {
			try {

                if (containingAtom.type != "CustomUnityAsset")
                {
                    SuperController.LogError("EnjoyGlassBody - Not a usable environment." + containingAtom.type);
                    return;
                }

                _initLoops = 0;

                _trigger_control = FeelUtil.FindWithinPlugin<FeelTriggersControl>(this);

                pose_triggers = new List<FeelTrigger>();
                
                for(int i=0; i<5; i++)
                {
                    FeelTrigger trigger = _trigger_control.CreateFeelTrigger(this, "Pose" + (i + 1) + "Trigger");
                    pose_triggers.Add(trigger);
                    JSONStorableAction pose_action = new JSONStorableAction("PoseAction" + i, () => {
                        trigger.OpenStartActionsPanel();
                    });
                    UIDynamicButton db = CreateButton("Pose" + (i + 1));
                    pose_action.dynamicButton = db;
                }

                _pov_trigger = _trigger_control.CreateFeelTrigger(this, "POVTrigger");
                JSONStorableAction pov_action = new JSONStorableAction("POVAction", () => {
                    _pov_trigger.OpenStartActionsPanel();
                });
                UIDynamicButton db1 = CreateButton("POV", true);
                pov_action.dynamicButton = db1;

                _start_trigger = _trigger_control.CreateFeelTrigger(this, "StartTrigger");
                JSONStorableAction start_action = new JSONStorableAction("StartAction", () => {
                    _start_trigger.OpenStartActionsPanel();
                });
                UIDynamicButton db2 = CreateButton("Start", true);
                start_action.dynamicButton = db2;

                _end_trigger = _trigger_control.CreateFeelTrigger(this, "EndTrigger");
                JSONStorableAction end_action = new JSONStorableAction("EndAction", () => {
                    _end_trigger.OpenStartActionsPanel();
                });
                UIDynamicButton db3 = CreateButton("End", true);
                end_action.dynamicButton = db3;

                _vr_on_trigger = _trigger_control.CreateFeelTrigger(this, "VROnTrigger");
                JSONStorableAction vr_on_action = new JSONStorableAction("VROnAction", () => {
                    _vr_on_trigger.OpenStartActionsPanel();
                });
                UIDynamicButton db4 = CreateButton("VROn", true);
                vr_on_action.dynamicButton = db4;
                _vr_off_trigger = _trigger_control.CreateFeelTrigger(this, "VROffTrigger");
                JSONStorableAction vr_off_action = new JSONStorableAction("VROffAction", () => {
                    _vr_off_trigger.OpenStartActionsPanel();
                });
                UIDynamicButton db5 = CreateButton("VROff", true);
                vr_off_action.dynamicButton = db5;

            }
            catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		// Start is called once before Update or FixedUpdate is called and after Init()
		void Start() {
			try {
				// put code in here
			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		// Update is called with each rendered frame by Unity
		void Update() {
			try {
				// put code in here
                if(_initLoops < _maxInitLoops)
                {
                    _initLoops++;
                    firstInit();
                }
                else
                {
                    if (_initLoops == 1001)
                    {
                        SuperController.LogError("EnjoyGlassBody Load Failure " + _initLoops);
                        _initLoops += 1;
                    }
                }

                if(_enjoyGlassBodyUI != null)
                {
                    _enjoyGlassBodyUI.UIUpdate();

                    if (Input.GetKeyDown(KeyCode.F5))
                    {
                        _enjoyGlassBodyUI.MainHUbShowToggle();
                    }
                }
            }
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		// FixedUpdate is called with each physics simulation frame by Unity
		void FixedUpdate() {
			try {
				// put code in here
			}
			catch (Exception e) {
				SuperController.LogError("Exception caught: " + e);
			}
		}

		// OnDestroy is where you should put any cleanup
		// if you registered objects to supercontroller or atom, you should unregister them here
		void OnDestroy() {
            _enjoyGlassBodyUI.OnCanvasDestory();
		}

        private void firstInit()
        {
            ObjectRoot = getObjectRoot(containingAtom.reParentObject, "EnjoyGlassBody");

            if (ObjectRoot != null && ObjectRoot.GetComponent<Canvas>() != null)
            {
                List<string> ids = SuperController.singleton.GetAtomUIDs();
                bool is_girl_ready = false;
                bool is_glassbody_ready = false;

                foreach (string id in ids)
                {
                    if (id == "Girl")
                    {
                        is_girl_ready = true;
                        _girl_atom = SuperController.singleton.GetAtomByUid(id);
                    }
                    if (id == "GlassBody") is_glassbody_ready = true;

                    if (id == "UIButton")
                    {
                        Atom atom = SuperController.singleton.GetAtomByUid(id);

                        for (int i = 0; i < atom.transform.childCount; i++)
                        {
                            //SuperController.LogMessage("-->> " + atom.transform.GetChild(i).name);
                            if (atom.transform.GetChild(i).name == "HolderCanvas")
                            {
                                Transform c_atom = atom.transform.GetChild(i);
                                Canvas ca = c_atom.GetComponent<Canvas>().rootCanvas;

                            }
                        }

                    }
                }
                if (!is_girl_ready || !is_glassbody_ready)
                {
                    SuperController.LogError("EnjoyGlassBody - Not ready.");
                    return;
                }

                _initLoops += _maxInitLoops;

                _enjoyGlassBodyUI = new EnjoyGlassBodyUI();
                _enjoyGlassBodyUI.SetMainCanvas(ObjectRoot.GetComponent<Canvas>(), OnSelectPose, OnPOVAction, OnStartAction, OnEndAction, SetVROnOff);


                SuperController.LogMessage("Init Complete.");
            }

            

        }

        private void OnSelectPose(int idx)
        {
            FeelTrigger trigger = pose_triggers[idx];
            trigger.Trigger();
        }

        private void OnPOVAction()
        {
            _pov_trigger.Trigger();
        }

        private void OnStartAction()
        {
            //SuperController.LogMessage("OnStartAction.");
            _start_trigger.Trigger();
        }

        private void OnEndAction()
        {
            //SuperController.LogMessage("OnEndAction.");
            _end_trigger.Trigger();
        }

        private void SetVROnOff(bool is_vr_on)
        {
            if (is_vr_on)
                _vr_on_trigger.Trigger();
            else
                _vr_off_trigger.Trigger();
        }

        private Transform getObjectRoot(Transform parent, string rootName)
        {
            foreach (Transform child in parent)
            {
                if (child.name.Contains(rootName))
                {
                    return child;
                }
                else
                {
                    Transform childSearch = getObjectRoot(child, rootName);
                    if (childSearch != null) return childSearch;
                }
            }

            return null;
        }

        private Transform getChildRoot(Transform parent, string searchName)
        {
            foreach (Transform child in parent)
            {
                if (child.name == searchName)
                {

                    return child;
                }
                else
                {
                    Transform childSearch = getChildRoot(child, searchName);
                    if (childSearch != null) return childSearch;
                }
            }

            return null;
        }

    }
}