﻿using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using UnityEngine.UI;
using UnityEngine.XR;

namespace Feel
{
	public class EnjoyGlassBodyUI {

        private Canvas _main_canvas;
        private Transform _main_panel;
        private Button _menu_btn;
        private Button _start_btn;
        private Button _end_btn;
        private Button _pov_btn;
        private Button _look_btn;
        private Button _new_ver_btn;
        private Dropdown _drop_down;

        private bool _is_hud_show = false;
        private bool _is_toggle = false;

        private bool _is_VR = false;
        private bool _is_main_show = false;

        private Action<int> _select_pose;
        private Action _pov_action;
        private Action _start_action;
        private Action _end_action;
        private Action<bool> _vr_onoff_action;

        public void SetMainCanvas(Canvas canvas, Action<int> select_pose, Action pov_action, Action start_action, Action end_action, Action<bool> vr_onoff_action)
        {
            _main_canvas = canvas;
            _select_pose = select_pose;
            _pov_action = pov_action;
            _start_action = start_action;
            _end_action = end_action;
            _vr_onoff_action = vr_onoff_action;

            try
            {
                RectTransform rect = _main_canvas.transform as RectTransform;
                
                _main_panel = _main_canvas.transform.Find("Panel").transform.Find("Panel");
                _is_VR = SuperController.singleton.isOVR;
                //_is_VR = true;
                if (_is_VR)
                {
                    _vr_onoff_action(true);
                    _main_canvas.renderMode = RenderMode.WorldSpace;
                    _main_canvas.worldCamera = Camera.main;

                    foreach (Camera camera in Camera.allCameras)
                    {
                        //SuperController.LogMessage(">> >> " + camera.name + " - " + camera.depth);

                        if (camera.name == "MonitorRig")
                        {
                            _main_canvas.worldCamera = camera;
                            //_main_canvas.sortingLayerID = -1000;

                        }
                    }

                    //_main_canvas.sortingLayerName = "AfterDefault";
                    //_main_canvas.sortingLayerID = -794000325;
                    //_main_canvas.sortingOrder = -2;

                    _main_canvas.transform.localScale = new Vector3(0.002f, 0.002f, 0.002f);
                    _main_canvas.transform.localPosition = new Vector3(0f, 0f, 0f);
                    _main_canvas.transform.localRotation = Quaternion.identity;

                    //SuperController.singleton.RightFullGrabbedController.CallAction("OnRightHandTrigger");
                    _main_panel.transform.localPosition = new Vector3(0f, 0f, 0f);

                    _main_panel.transform.Find("GuideDescription").gameObject.SetActive(false);
                }
                else
                {
                    _vr_onoff_action(false);
                    _main_canvas.renderMode = RenderMode.ScreenSpaceOverlay;
                    _main_canvas.transform.localScale = new Vector3(1f, 1f, 1f);
                    _main_canvas.transform.localPosition = new Vector3(0f, 0f, 0f);

                    //_main_panel.transform.Find("GuideDescription").GetComponent<Text>().color = Color.white;
                }

                SuperController.singleton.AddCanvas(canvas);
                IgnoreCanvas IgnoreCanvas = canvas.gameObject.AddComponent<IgnoreCanvas>();

                _menu_btn = _main_panel.transform.Find("MenuBtn").GetComponent<Button>();
                _menu_btn.onClick.AddListener(OnMenuBtnClick);
                _start_btn = _main_panel.transform.Find("StartBtn").GetComponent<Button>();
                _start_btn.onClick.AddListener(OnStartBtnClick);
                _end_btn = _main_panel.transform.Find("StopBtn").GetComponent<Button>();
                _end_btn.onClick.AddListener(OnStopBtnClick);
                _new_ver_btn = _main_panel.transform.Find("NewVerBtn").GetComponent<Button>();
                _new_ver_btn.onClick.AddListener(OnNewVerBtnClick);
                _look_btn = _main_panel.transform.Find("LookBtn").GetComponent<Button>();
                _look_btn.onClick.AddListener(OnStopBtnClick);
                _pov_btn = _main_panel.transform.Find("POVBtn").GetComponent<Button>();
                _pov_btn.onClick.AddListener(OnPOVBtnClick);
                _drop_down = _main_panel.transform.Find("Dropdown").GetComponent<Dropdown>();
                List<Dropdown.OptionData> pose_list = new List<Dropdown.OptionData>();

                for (int i = 0; i < 5; i++)
                {
                    Dropdown.OptionData option = new Dropdown.OptionData("POSE" + (i + 1));
                    pose_list.Add(option);
                }

                _drop_down.options = pose_list;
                _drop_down.onValueChanged.AddListener(delegate {
                    OnDropdownValueChanged();
                });


                _drop_down.gameObject.SetActive(false);
                _look_btn.gameObject.SetActive(false);

                _main_panel.transform.Find("Slider").gameObject.SetActive(false);
            }
            catch(Exception e)
            {
                SuperController.LogError("EnjoyGlassBodyUI SetMainCanvas Exception \n" + e.Message);
            }
            
            

        }

        public void UIUpdate()
        {
            if(_main_canvas != null)
            {
                if (!_is_toggle)
                {
                    bool is_hud_show = false;
                    if (SuperController.singleton.templatesFileBrowserWorldUI.isActiveAndEnabled)
                        is_hud_show = false;
                    else
                        is_hud_show = true;

                    if (_is_hud_show != is_hud_show)
                    {
                        _main_canvas.gameObject.SetActive(is_hud_show);
                        _is_hud_show = is_hud_show;
                    }

                }

                if (_is_VR)
                {
                    if(SuperController.singleton.mainHUD.gameObject.activeSelf)
                    {
                        if(!_is_main_show)
                        {
                            _is_main_show = true;
                            _main_canvas.transform.position = SuperController.singleton.rightHand.position + new Vector3(0f, 0f, -1f);
                            _main_canvas.transform.rotation = SuperController.singleton.rightHand.rotation;
                        }
                    }
                    else
                    {
                        if(_is_main_show)
                        {
                            _is_main_show = false;
                        }
                    }

                    _main_canvas.gameObject.SetActive(_is_main_show);
                    //_main_canvas.transform.position = SuperController.singleton.rightHand.position;
                    //_main_canvas.transform.rotation = SuperController.singleton.rightHand.rotation;

                    //_main_canvas.transform.position = SuperController.singleton.mainHUD.position + new Vector3(-1f, -1f, -1f);
                    //_main_canvas.transform.rotation = SuperController.singleton.mainHUD.rotation;
                }
            }
            
        }

        public void MainHUbShowToggle()
        {
            if (_main_canvas != null)
            {
                if (_is_toggle)
                    _is_toggle = false;
                else
                    _is_toggle = true;

                _main_canvas.gameObject.SetActive(!_is_toggle);
                
            }
        }

        public void OnCanvasDestory()
        {
            SuperController.singleton.RemoveCanvas(_main_canvas);
        }

        private void OnRightHandTrigger()
        {
            SuperController.LogMessage("-->> OnRightHandTrigger");
        }

        private void OnDropdownValueChanged()
        {
            Dropdown.OptionData data = _drop_down.options[_drop_down.value];
            //SuperController.LogMessage(">> OnDropdownValueChanged " + data.text);
            _select_pose(_drop_down.value);
        }

        private void OnMenuBtnClick()
        {
            _main_canvas.sortingLayerName = "AfterDefault";
            _main_canvas.sortingLayerID = -794000325;
            //_main_canvas.sortingOrder = -2;

            if (_drop_down.gameObject.activeSelf)
                _drop_down.gameObject.SetActive(false);
            else
                _drop_down.gameObject.SetActive(true);
        }

        private void OnStartBtnClick()
        {
            _start_action();
        }

        private void OnStopBtnClick()
        {
            _end_action();
        }

        private void OnPOVBtnClick()
        {
            _pov_action();
        }

        private void OnNewVerBtnClick()
        {
            //SuperController.LogMessage(">> OnNewVerBtnClick ");
            Application.OpenURL("https://www.patreon.com/feel_vam");
        }
    }
}